=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-08-09 07:52:25 +0000
+++ bzrlib/bzrdir.py	2006-08-11 05:22:34 +0000
@@ -685,8 +685,7 @@
         # done on this format anyway. So - acceptable wart.
         result = self.open_workingtree()
         if revision_id is not None:
-            result.set_parent_trees([(revision_id,
-                result.branch.repository.revision_tree(revision_id))])
+            result.set_parent_tree_ids([revision_id])
         return result
 
     def get_branch_transport(self, branch_format):

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2006-07-26 07:11:35 +0000
+++ bzrlib/merge.py	2006-08-11 05:22:34 +0000
@@ -181,7 +181,7 @@
         ancestry = self.this_branch.repository.get_ancestry(self.this_basis)
         if self.other_rev_id in ancestry:
             return
-        self.this_tree.add_pending_merge(self.other_rev_id)
+        self.this_tree.add_parent_tree((self.other_rev_id, self.other_tree))
 
     def set_other(self, other_revision):
         other_branch, self.other_tree = _get_tree(other_revision, 

=== modified file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/tests/test_merge_core.py	2006-08-09 07:52:25 +0000
+++ bzrlib/tests/test_merge_core.py	2006-08-11 05:22:34 +0000
@@ -53,8 +53,7 @@
         for tt, wt in ((self.this_tt, self.this), (self.other_tt, self.other)):
             # why does this not do wt.pull() ?
             wt.branch.pull(self.base.branch)
-            wt.set_parent_trees([(wt.branch.last_revision(),
-                wt.branch.repository.revision_tree(wt.branch.last_revision()))])
+            wt.set_parent_ids([wt.branch.last_revision()])
             tt.apply()
             wt.commit('branch commit')
             assert len(wt.branch.revision_history()) == 2

=== modified file 'bzrlib/tests/workingtree_implementations/test_basis_inventory.py'
--- bzrlib/tests/workingtree_implementations/test_basis_inventory.py	2006-08-09 07:52:25 +0000
+++ bzrlib/tests/workingtree_implementations/test_basis_inventory.py	2006-08-11 05:22:34 +0000
@@ -59,7 +59,7 @@
     def test_basis_inv_gets_revision(self):
         """When the inventory of the basis tree has no revision id it gets set.
 
-        It gets set during set_parent_trees()
+        It gets set during set_parent_trees() or set_parent_ids().
         """
         tree = self.make_branch_and_tree('.')
         tree.lock_write()

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-08-09 07:52:25 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-08-11 05:22:34 +0000
@@ -351,12 +351,10 @@
         wt.commit('A', rev_id='A')
         wt.rename_one('foo', 'bar')
         wt.commit('B', rev_id='B')
-        wt.set_parent_trees(
-            [('B', wt.branch.repository.revision_tree('B'))])
+        wt.set_parent_ids(['B'])
         tree = wt.basis_tree()
         self.failUnless(tree.has_filename('bar'))
-        wt.set_parent_trees(
-            [('A', wt.branch.repository.revision_tree('A'))])
+        wt.set_parent_ids(['A'])
         tree = wt.basis_tree()
         self.failUnless(tree.has_filename('foo'))
 

=== modified file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	2006-08-09 07:52:25 +0000
+++ bzrlib/uncommit.py	2006-08-11 05:22:34 +0000
@@ -80,10 +80,7 @@
                 else:
                     parents = []
                 parents.extend(reversed(pending_merges))
-                parent_trees = [
-                    (parent, branch.repository.revision_tree(parent))
-                    for parent in parents]
-                tree.set_parent_trees(parent_trees)
+                tree.set_parent_ids(parents)
     finally:
         for item in reversed(unlockable):
             item.unlock()

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-08-10 07:06:53 +0000
+++ bzrlib/workingtree.py	2006-08-11 05:22:34 +0000
@@ -520,8 +520,7 @@
             # TODO now merge from tree.last_revision to revision (to preserve
             # user local changes)
             transform_tree(tree, self)
-            tree.set_parent_trees([(revision_id,
-                tree.branch.repository.revision_tree(revision_id))])
+            tree.set_parent_ids([revision_id])
 
     @needs_write_lock
     def commit(self, message=None, revprops=None, *args, **kwargs):
@@ -1056,7 +1055,12 @@
                 finally:
                     pb.finished()
                 # TODO - dedup parents list with things merged by pull ?
+                # reuse the revisiontree we merged against to set the new
+                # tree data.
                 parent_trees = [(self.branch.last_revision(), new_basis_tree)]
+                # we have to pull the merge trees out again, because 
+                # merge_inner has set the ids. - this corner is not yet 
+                # layered well enough to prevent double handling.
                 merges = self.get_parent_ids()[1:]
                 parent_trees.extend([
                     (parent, repository.revision_tree(parent)) for
@@ -1476,8 +1480,12 @@
                                       this_tree=self)
                 self.set_last_revision(self.branch.last_revision())
                 # TODO - dedup parents list with things merged by pull ?
+                # reuse the tree we've updated to to set the basis:
                 parent_trees = [(self.branch.last_revision(), to_tree)]
                 merges = self.get_parent_ids()[1:]
+                # pull the other trees out of the repository. This could be
+                # better expressed - for instance by inserting a parent, and
+                # that would remove duplication.
                 parent_trees.extend([
                     (parent, self.branch.repository.revision_tree(parent)) for
                      parent in merges])

